/*  FreeJ
 *  (c) Copyright 2001 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <unistd.h>
#include <sys/time.h>
#include <assert.h>
#include <context.h>
#include <config.h>

/* flags can be:
   SDL_ANYFORMAT
   SDL_HWPALETTE
   SDL_DOUBLEBUF
   SDL_FULLSCREEN */

Context::Context(int wx, int hx, int bppx, Uint32 flags) {
  int res;
  surf = NULL;
  cerr << "-- Context::Context (using Simple Direct Media Layer)\n";
  
  if( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    cerr << "   Can't initialize SDL: " << SDL_GetError() << endl;
    exit(1);
  }

  assert( res = SDL_VideoModeOK(wx, hx, bppx, flags) );

  if(res!=16) {
    cerr << "   your screen does'nt support 16bit depth resolution";
    cerr << "   check your Xserver configuration.";
    return;
  }

  fprintf(stderr,"   initializing w[%u] h[%u] bpp[%u]...",wx,hx,res);

  surf = SDL_SetVideoMode(wx, hx, res, flags);
  if( surf == NULL ) {
    fprintf(stderr,"\n   can't set w[%u] h[%u] bpp[%u] video mode: %s\n",
	    wx, hx, res, SDL_GetError());
    exit(1);
  }

  w = wx; h = hx;
  bpp = surf->format->BitsPerPixel;
  size = w*h*(bpp>>3);
  pitch = w*(bpp>>3);
  
  /* precalculate coords tables */
  for(int c=0;c<h;c++)
    prec_y[c] = (Uint16 *)surf->pixels + (w*c);
  
  /* be nice with the window manager */
  char temp[120];
  sprintf(temp,"%s %s",PACKAGE,VERSION);
  SDL_WM_SetCaption (temp, temp);
  
  /* ignore events :: only keys and wm_quit */
  for ( int i=SDL_NOEVENT; i<SDL_NUMEVENTS; ++i )
    if( (i != SDL_KEYDOWN) && ( i != SDL_QUIT) )
      SDL_EventState(i, SDL_IGNORE);

  SDL_ShowCursor(0);

  /* initialize fps counter */
  fps_frame_interval = -1;

  cerr << " OK!" << endl;

}  

Context::~Context() {
  cerr << "-- Context::~Context\n";
  SDL_Quit();
  cerr << "   clean exiting. be nice." << endl;
}

bool Context::flip() {
  SDL_Flip(surf);
  return(true);
}

unsigned char *Context::get_surface() {
  return((unsigned char *)surf->pixels);
}

/* FPS */

double Context::_time() {
  struct timeval tv;
  double curtime;
  
  /* get wallclock time */
  gettimeofday(&tv, NULL);
  curtime=(double)tv.tv_sec+1.0e-6*(double)tv.tv_usec;
  return (curtime);
}

void Context::set_fps_interval(int interval) {
  fps_frame_interval = interval;
  framecount = -1;
}

bool Context::calc_fps() {
  double curtime;
  
  /* Check interval */
  if (fps_frame_interval<1) {
    fps=0.0;
    return(true);
  } else {
    if ( (framecount<0) || (framecount>=fps_frame_interval)) {
      /* Initialize counter */
      framecount=0;
      lst_time=_time();
      fps=0.0;
      return(true);
    } else {
      framecount++;
      if (framecount==fps_frame_interval) {
	curtime=_time();
	fps=(double)framecount/(curtime-lst_time);
	lst_time=curtime;
	framecount=0;
	return(false);
      }  
    }
  }  
  return(true);
}

void Context::print_fps() {
  fprintf(stderr,"Layer:: fps [%f]\n",fps);
}
