/*
  FreeJ effect Delaygrab

  delaygrab - blockwise, controllable image delay
  Copyright (C) 1999/2000  A. Schiffler
  
  the original sourcecode is from libbgrab 2.1f
  ported to FreeJ by jaromil
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  To contact the author try these:
  |  Andreas Schiffler                    aschiffler@home.com  |
  |  Senior Systems Engineer    -    Deskplayer Inc., Buffalo  |
  |  4707 Eastwood Cres., Niagara Falls, Ont  L2E 1B4, Canada  |  
  |  +1-905-371-3652 (private)  -  +1-905-371-8834 (work/fax)  |
  
*/

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>

#include "delaygrab.h"

#define BLOCKSIZE 2
#define QUEUEDEPTH 76

Delaygrab::Delaygrab() {
  initialized = false;
  delaymap=NULL;
  imagequeue=NULL;
  procbuf=NULL;
}

Delaygrab::~Delaygrab() {
  cerr << "-- Delaygrab::~Delaygrab\n";
  if(delaymap!=NULL) {
    fprintf(stderr,"   freeing delaymap[0x%lx]\n",(unsigned long int)delaymap);
    free(delaymap);
  }
  if(imagequeue!=NULL) {
    fprintf(stderr,"   freeing imagequeue[0x%lx]\n",(unsigned long int)imagequeue);
    free(imagequeue);
  }
  if(procbuf!=NULL) {
    fprintf(stderr,"   freeing procbuf[0x%lx]\n",(unsigned long int)procbuf);
    free(procbuf);
  }
}

bool Delaygrab::init() {
  cerr << "-- Delaygrab::init\n";

  delaymapwidth = (*w)/BLOCKSIZE;
  delaymapheight = (*h)/BLOCKSIZE;
  delaymapsize = delaymapheight*delaymapwidth;
  fprintf(stderr,"   delaymapwidth[%u] delaymapheight[%u] delaymapsize[%u]\n",delaymapwidth,delaymapheight,delaymapsize);
  cerr << "   seeding randomizer\n";
  srand(time(NULL));

  imagequeue=(unsigned char *)malloc(QUEUEDEPTH*(*size));
  if(imagequeue==NULL) {
    fprintf(stderr," ! Not enough memory for a %i frame queue (%i bytes)\n",QUEUEDEPTH,QUEUEDEPTH*(*size));
    return(false);
  } else
    fprintf(stderr,"   allocated imagequeue[0x%lx] sized[%u]\n",(unsigned long int)imagequeue,QUEUEDEPTH*(*size));

  procbuf=(unsigned char *)malloc(*size);
  if(procbuf==NULL) {
    fprintf(stderr," ! out of memory allocating procbuf sized[%u]\n",*size);
    return(false);
  } else
    fprintf(stderr,"   allocated procbuf[0x%lx] sized[%u]\n",(unsigned long int)procbuf,*size);
  
  curqueue=imagequeue;
  curqueuenum=0;
  linewidth = (*w) *(bpp>>3);

  createDelaymap(2);
    
  return(true);
}

void Delaygrab::createDelaymap(int mode) {
  cerr << "-- Delaygrab::createDelaymap\n";
  
  if(delaymap==NULL) {
    delaymap=(int *)malloc(delaymapsize*sizeof(int));
    fprintf(stderr,"   allocated delaymap[0x%lx] sized[%lu] for mode[%u]\n",(unsigned long int)delaymap,(unsigned long int)delaymapsize*sizeof(int),mode);
  }

  curdelaymap=delaymap;
  for (y=0; y<delaymapheight; y++) {
  for (x=0; x<delaymapwidth; x++) {
   switch (mode) {
   case 1:
     double d;
     /* Random delay with square distribution */
     d = (double)rand()/(double)RAND_MAX;
     *curdelaymap = (int)(d*d*16.0);
     break;
    case 2:
     /* Stripes of increasing delay outward from center */
     if (x<(delaymapwidth/2)) {
      v=(delaymapwidth/2)-x;
     } else if (x>(delaymapwidth/2)) {
      v=x-(delaymapwidth/2);
     } else {
      v=0;
     }
     *curdelaymap=v/2;
     break;
    case 3:
     /* Rings of increasing delay outward from center */
     v = (int)sqrt((double)((x-(delaymapwidth/2))*(x-(delaymapwidth/2))+(y-(delaymapheight/2))*(y-(delaymapheight/2))));
     *curdelaymap=v/2;
     break;
   } // switch
   /* Clip values */
   if (*curdelaymap<0) {
    *curdelaymap=0;
   } else if (*curdelaymap>(QUEUEDEPTH-1)) {
    *curdelaymap=(QUEUEDEPTH-1);
   }
   curdelaymap++;
  }
 }
}

unsigned char *Delaygrab::process(unsigned char *buffo) {

  /* Update queue pointer */
  if (curqueuenum==0) {
    curqueuenum=QUEUEDEPTH-1;
    curqueue = imagequeue;
    curqueue += (*size*(QUEUEDEPTH-1));
  } else {
    curqueuenum--;
    curqueue -= *size;
  }

   /* Copy image to queue */
  mmxcopy(buffo,curqueue,*size);

     /* Copy image blockwise to screenbuffer */
  curdelaymap=delaymap;
  for (y=0; y<delaymapheight; y++) {
    for (x=0; x<delaymapwidth; x++) {
     curposnum=((curqueuenum + (*curdelaymap)) % QUEUEDEPTH);
     xyoff= x*(BLOCKSIZE*(bpp>>3)) + y*(BLOCKSIZE*linewidth);
     // source
     curpos= (unsigned char *)imagequeue;
     curpos += (*size*curposnum);
     curpos += xyoff;
     // target
     curimage = (unsigned char *)procbuf;
     curimage += xyoff;
     // copy block
     for (i=0; i<BLOCKSIZE; i++) {
       memcpy((unsigned char *)curimage,(unsigned char *)curpos,(BLOCKSIZE<<1));
       curpos += (linewidth);
       curimage += (linewidth);
     }
     curdelaymap++;
    }
   }

  return(procbuf);
}
